﻿Imports System.Drawing.Printing
Imports System.Drawing.Drawing2D

Public Class Form1
    ' Display a print preview dialog.
    Private Sub btnPrintPreview_Click() Handles btnPrintPreview.Click
        dlgPrintPreview.ShowDialog()
    End Sub

    ' Display a print dialog.
    Private Sub btnPrintDialog_Click() Handles btnPrintDialog.Click
        dlgPrint.ShowDialog()
    End Sub

    ' Print now.
    Private Sub btnPrintNow_Click() Handles btnPrintNow.Click
        pdocGraph.Print()
    End Sub

    ' Print a page.
    Private Sub pdocGraph_PrintPage(ByVal sender As Object, ByVal e As System.Drawing.Printing.PrintPageEventArgs) Handles pdocGraph.PrintPage
        ' Draw the margins (for debugging). Be sure 
        ' to do this before transforming the Graphics object.
        e.Graphics.DrawRectangle(Pens.Red, e.MarginBounds)

        ' This routine draws a bar chart for 5 values
        ' in printer coordinates between 
        ' (100, 100) - (600, 400). 
        ' Transform the Graphics object to center the results.
        Dim picture_rect As New RectangleF(100, 100, 500, 300)
        Dim margin_rect As New RectangleF( _
            e.MarginBounds.X, _
            e.MarginBounds.Y, _
            e.MarginBounds.Width, _
            e.MarginBounds.Height)
        FitPictureToMargins(e.Graphics, picture_rect, margin_rect)

        ' Draw a rectangle around the chart.
        e.Graphics.FillRectangle(Brushes.LightGray, picture_rect)
        e.Graphics.DrawRectangle(Pens.Black, Rectangle.Round(picture_rect))

        ' Draw the values.
        Dim x As Integer = 100
        DrawBar(e.Graphics, x, 200, HatchStyle.BackwardDiagonal)
        DrawBar(e.Graphics, x, 280, HatchStyle.Vertical)
        DrawBar(e.Graphics, x, 240, HatchStyle.ForwardDiagonal)
        DrawBar(e.Graphics, x, 170, HatchStyle.Horizontal)
        DrawBar(e.Graphics, x, 290, HatchStyle.DiagonalCross)

        ' There are no more pages.
        e.HasMorePages = False
    End Sub

    ' Draw a bar in (x, 400)-(x + 100, 400 - hgt).
    Private Sub DrawBar(ByVal gr As Graphics, ByRef x As Integer, ByVal hgt As Integer, ByVal hatch_style As HatchStyle)
        Dim rect As New Rectangle(x, 400 - hgt, 100, hgt)
        Using hatch_brush As New HatchBrush(hatch_style, Color.Black, Color.White)
            gr.FillRectangle(hatch_brush, rect)
        End Using
        gr.DrawRectangle(Pens.Black, rect)
        x += 100
    End Sub

    ' Transform the Graphics object to center the rectangle
    ' picture_bounds within margin_bounds.
    Private Sub CenterPictureInMargins(ByVal gr As Graphics, ByVal picture_bounds As RectangleF, ByVal margin_bounds As RectangleF)
        ' Remove any existing transformation.
        gr.ResetTransform()

        ' Apply the transformation.
        Dim dx As Single = _
            margin_bounds.Left - picture_bounds.Left + _
            (margin_bounds.Width - picture_bounds.Width) / 2
        Dim dy As Single = _
            margin_bounds.Top - picture_bounds.Top + _
            (margin_bounds.Height - picture_bounds.Height) / 2
        gr.TranslateTransform(dx, dy)
    End Sub

    ' Transform the Graphics object to fit the rectangle
    ' picture_bounds to margin_bounds and center it.
    Private Sub FitPictureToMargins(ByVal gr As Graphics, ByVal picture_bounds As RectangleF, ByVal margin_bounds As RectangleF)
        ' Remove any existing transformation.
        gr.ResetTransform()

        ' Translate to center picture_bounds at the origin.
        gr.TranslateTransform( _
            -(picture_bounds.Left + picture_bounds.Width / 2), _
            -(picture_bounds.Top + picture_bounds.Height / 2))

        ' Scale to make picture_bounds fit margin_bounds.
        ' Compare the aspect ratios.
        Dim margin_aspect As Single = margin_bounds.Height / margin_bounds.Width
        Dim picture_aspect As Single = picture_bounds.Height / picture_bounds.Width
        Dim scale As Single
        If picture_aspect > margin_aspect Then
            ' picture_bounds is relatively tall and thin.
            ' Make it as tall as possible.
            scale = margin_bounds.Height / picture_bounds.Height
        Else
            ' picture_bounds is relatively short and wide.
            ' Make it as wide as possible.
            scale = margin_bounds.Width / picture_bounds.Width
        End If
        ' Scale.
        gr.ScaleTransform(scale, scale, MatrixOrder.Append)

        ' Translate to move the origin to the center of margin_bounds.
        gr.TranslateTransform( _
            margin_bounds.Left + margin_bounds.Width / 2, _
            margin_bounds.Top + margin_bounds.Height / 2, _
            MatrixOrder.Append)
    End Sub
End Class
